/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */
#include "config.h"
#include <X11/Xos.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/FileSB.h>
#include <Xm/Frame.h>
#include <Xm/MainW.h>
#include <Xm/RowColumn.h>
#include <Xm/LabelG.h>
#include <Xm/Form.h>
#include <Xm/List.h>
#include <Xm/Label.h>
#include <Xm/ScrollBar.h>
#include <Xm/DrawnB.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#ifdef HAVE_SYS_FILIO_H
#include <sys/filio.h>
#endif
#include <stdio.h>
#include <string.h>
#include <sys/param.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <setjmp.h>
#include <time.h>
#include "gps.h"
#include "xmap.h"
#include "debug.h"

extern XtAppContext app;
extern Widget toplevel;

struct gpsdata gpsdata =
{-1};
static jmp_buf env;

static XtInputId input_id;
static XtIntervalId id;

static void handle_message(char *message)
{
    char *p;
    struct tm *ttime;
    animate_button(1);

// fprintf(stderr,"\n%s\n",message);
    // This is what we expect:
    // GPSD,S=1,P=36.955648 -121.880762,D=98/08/31 19:59:14,A=0.000000,V=0.000000

    p = strtok(message, ",");

    if (p && (strncmp(p, "GPSD", 4) == 0)) {
	while (p = strtok(NULL, ",")) {
	    switch (*p) {
	    case 'S':
		sscanf(p, "S=%d", &(gpsdata.status));
		break;
	    case 'M':
		sscanf(p, "M=%d", &(gpsdata.mode));
		break;
	    case 'P':
		sscanf(p, "P=%lf %lf", &(gpsdata.latitude),
		       &(gpsdata.longitude));
		break;
	    case 'D':
		sscanf(p, "D=%s %s", gpsdata.date, gpsdata.time);

		gpsdata.gpstime = time(NULL);
		ttime = gmtime(&gpsdata.gpstime);
		sscanf(p, "D=%d/%d/%d %d:%d:%d",
		       &ttime->tm_mon, &ttime->tm_mday, &ttime->tm_year,
		       &ttime->tm_hour, &ttime->tm_min, &ttime->tm_sec);
		ttime->tm_year -= 1900;
		ttime->tm_mon -= 1;
		/* Was timegm -- why? */
		gpsdata.gpstime = mktime(ttime);
		break;
	    case 'A':
		sscanf(p, "A=%lf", &(gpsdata.altitude));
		break;
	    case 'V':
		sscanf(p, "V=%lf", &(gpsdata.velocity));
		break;
	    }
	}
    }
}

static void handle_input(XtPointer client_data, int *source, XtInputId id)
{
    static char bufp[256];	/* that is more then a sentence */
    static int offset = 0;
    int count;

    ioctl(*source, FIONREAD, &count);	// this is BAD! Why can't I make the
    // socket non blocking for read?

    while (offset < 255 && count--) {
	if (read(*source, bufp + offset, 1) != 1)
	    return;
	if (bufp[offset] == '\n') {
	    if (bufp[offset - 1] == '\r')
		bufp[offset - 1] = '\0';
	    handle_message(bufp);
	    offset = 0;
	    return;
	}
	offset++;
    }
    offset = 0;			/* discard input ! */
}

static void to_cb(Widget w)
{
    long interval = GetPrivateProfileInt("General", 
					 "gpsinterval", 1000L, inifile);
    id = XtAppAddTimeOut(app, interval, (XtTimerCallbackProc) to_cb, w);
    if (write(gpsdata.fd, "SPDAVM\n", 6) != 6) {
	    gps_unconnect();
	    fprintf(stderr,"Lost connection to GPS server!\n");
    }
    else {
	do_log();
	do_track();
    }
}

static int attached = 0;

attach_gps_input()
{
    input_id = XtAppAddInput(app, gpsdata.fd, (XtPointer) XtInputReadMask,
			     (XtInputCallbackProc) handle_input, NULL);

    id = XtAppAddTimeOut(app, 100L, (XtTimerCallbackProc) to_cb, toplevel);

    attached = 1;
}

detach_gps_input()
{
    if (attached) {
	XtRemoveTimeOut(id);
	XtRemoveInput(input_id);

	attached = 0;
    }
}

int gps_connect()
{
    char temp[255];
    char *p;
    int err;
    int one = 1;
    char *port = DEFAULTPORT;

    GetPrivateProfileString("Servers", "server1", "", temp, 128, inifile);

    if (!*temp) {
	error(
		 "You have not choosen a Server!\nGo to Edit/Preferences/GPS and\nselect a Server."
		 ,0);
	return CONNECT_FAILED;
    }
    if (p = strchr(temp, ':')) {
	if (*(p + 1))
	    port = p + 1;
	*p = '\0';
    }
    if (setjmp(env)) {
	gps_unconnect();
	return CONNECT_FAILED;
    }
    //temp now holds the HOSTNAME portion and port the port number.
    gpsdata.fd = connectTCP(temp, port);

    if (gpsdata.fd == -1)
	return CONNECT_FAILED;

    setsockopt(gpsdata.fd, SOL_SOCKET, SO_REUSEADDR, (char *) &one, sizeof(one));

    gpsdata.status = 0;

    attach_gps_input();
    return gpsdata.fd;
}

int gps_unconnect()
{
    if (gpsdata.fd != -1) {
	detach_gps_input();
	close(gpsdata.fd);
	gpsdata.fd = -1;
    }
    gpsdata.status = -1;
    animate_button(0);
    return UNCONNECTED;
}

void io_error()
{
    longjmp(env, 1);
}

int gps_valid()
{
    if (gpsdata.status > 0 && gpsdata.mode > 1)
	return TRUE;
    return FALSE;
}
