/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */

#include "config.h"
#include <Xm/MainW.h>
#include <Xm/PanedW.h>
#include <Xm/RowColumn.h>
#include <Xm/DrawingA.h>
#include <Xm/CascadeBG.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/ScrolledW.h>
#include <stdio.h>
#include "xmap.h"
#include "debug.h"

Widget
BuildMenu(parent, menu_type, menu_title, menu_mnemonic, items)
Widget parent;
int menu_type;
char *menu_title, menu_mnemonic;
MenuItem *items;
{
    Widget menu, cascade, widget;
    int i;
    XmString str;

    if (menu_type == XmMENU_PULLDOWN || menu_type == XmMENU_OPTION) {
	menu = XmCreatePulldownMenu(parent, menu_title, NULL, 0);
    } else if (menu_type == XmMENU_POPUP)
	menu = XmCreatePopupMenu(parent, menu_title, NULL, 0);
    else if (menu_type == XmMENU_BAR)
	menu = XmCreateMenuBar(parent, menu_title, NULL, 0);
    else {
	XtWarning("Invalid menu type passed to BuildMenu()");
	return NULL;
    }

    /* Pulldown menus require a cascade button to be made */
    if (menu_type == XmMENU_PULLDOWN) {
	cascade = XtVaCreateManagedWidget(menu_title,
				      xmCascadeButtonGadgetClass, parent,
					  XmNsubMenuId, menu,
					  NULL);
	if (menu_mnemonic) {
	    XtVaSetValues(cascade, XmNmnemonic, menu_mnemonic, NULL);
	}
    } else if (menu_type == XmMENU_OPTION) {
	/* Option menus are a special case, but not hard to handle */
	Arg args[2];
	str = XmStringCreateSimple(menu_title);
	XtSetArg(args[0], XmNsubMenuId, menu);
	XtSetArg(args[1], XmNlabelString, str);
	/* This really isn't a cascade, but this is the widget handle
	 * we're going to return at the end of the function.
	 */
	cascade = XmCreateOptionMenu(parent, menu_title, args, 2);
	XmStringFree(str);
    }
    /* Now add the menu items */
    for (i = 0; items[i].label != NULL; i++) {
	/* If subitems exist, create the pull-right menu by calling this
	 * function recursively.  Since the function returns a cascade
	 * button, the widget returned is used..
	 */
	if (items[i].subitems)
	    if (menu_type == XmMENU_OPTION) {
		XtWarning("You can't have submenus from option menu items.");
		continue;
	    } else
		widget = items[i].widget = BuildMenu(menu, XmMENU_PULLDOWN,
		   items[i].label, items[i].mnemonic, items[i].subitems);
	else {
	    widget = items[i].widget = XtVaCreateManagedWidget(items[i].label,
						   *items[i].class, menu,
							       NULL);
	}

	/* Whether the item is a real item or a cascade button with a
	 * menu, it can still have a mnemonic.
	 */
	if (items[i].mnemonic)
	    XtVaSetValues(widget, XmNmnemonic, items[i].mnemonic, NULL);

	/* any item can have an accelerator, except cascade menus. But,
	 * we don't worry about that; we know better in our declarations.
	 */
	if (items[i].accelerator) {

//fprintf(stderr,"Loading accel \"%s\", text \"%s\"\n",items[i].accelerator,items[i].accel_text);
	    str = XmStringCreateSimple(items[i].accel_text);
	    XtVaSetValues(widget,
			  XmNaccelerator, items[i].accelerator,
			  XmNacceleratorText, str,
			  NULL);
	    XmStringFree(str);
	}

	if (items[i].callback)
	    XtAddCallback(widget,
			  (items[i].class == &xmToggleButtonWidgetClass ||
			   items[i].class == &xmToggleButtonGadgetClass) ?
			  XmNvalueChangedCallback :	/* ToggleButton class */
			  XmNactivateCallback,	/* PushButton class */
			  items[i].callback, items[i].callback_data);
    }

    /* for popup menus, just return the menu; pulldown menus, return
     * the cascade button; option menus, return the thing returned
     * from XmCreateOptionMenu().  This isn't a menu, or a cascade button!
     */
    if (menu_type == XmMENU_BAR)
	return menu;
    return menu_type == XmMENU_POPUP ? menu : cascade;
}

/* Pulldown menus are built from cascade buttons, so this function
 * also includes pullright menus.  Create the menu, the cascade button
 * that owns the menu, and then the submenu items.
 */
Widget
BuildPulldownMenu(parent, menu_title, menu_mnemonic, tear_off, items)
Widget parent;
char *menu_title, menu_mnemonic;
Boolean tear_off;
MenuItem *items;
{
    Widget PullDown, cascade, widget;
    int i;
    XmString str;

    PullDown = XmCreatePulldownMenu (parent, "_pulldown", NULL, 0);
    if (tear_off)
        XtVaSetValues (PullDown, XmNtearOffModel, XmTEAR_OFF_ENABLED, NULL);
    str = XmStringCreateLocalized (menu_title);
    cascade = XtVaCreateManagedWidget (menu_title,
        xmCascadeButtonGadgetClass, parent,
        XmNsubMenuId,   PullDown,
        XmNlabelString, str,
        XmNmnemonic,    menu_mnemonic,
        NULL);
    XmStringFree (str);

    /* Now add the menu items */
    for (i = 0; items[i].label != NULL; i++) {
        /* If subitems exist, create the pull-right menu by calling this
         * function recursively.  Since the function returns a cascade
         * button, the widget returned is used..
         */
        if (items[i].subitems)
            widget = BuildPulldownMenu (PullDown, items[i].label, 
                items[i].mnemonic, tear_off, items[i].subitems);
        else
            widget = XtVaCreateManagedWidget (items[i].label,
                *items[i].class, PullDown,
                NULL);
        /* Whether the item is a real item or a cascade button with a
         * menu, it can still have a mnemonic.
         */
        if (items[i].mnemonic)
            XtVaSetValues (widget, XmNmnemonic, items[i].mnemonic, NULL);
        /* any item can have an accelerator, except cascade menus. But,
         * we don't worry about that; we know better in our declarations.
         */
        if (items[i].accelerator) {
            str = XmStringCreateLocalized (items[i].accel_text);
            XtVaSetValues (widget,
                XmNaccelerator, items[i].accelerator,
                XmNacceleratorText, str,
                NULL);
            XmStringFree (str);
        }
        if (items[i].callback)
            XtAddCallback(widget,
                (items[i].class == &xmToggleButtonWidgetClass ||
                items[i].class == &xmToggleButtonGadgetClass) ?
                    XmNvalueChangedCallback : /* ToggleButton class */
                    XmNactivateCallback,      /* PushButton class */
                items[i].callback, items[i].callback_data);
    }
    return cascade;
}

